package com.company.access.view.main;

import com.vaadin.flow.component.AbstractField.ComponentValueChangeEvent;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.html.IFrame;
import com.vaadin.flow.component.login.AbstractLogin;
import com.vaadin.flow.component.login.LoginOverlay;
import com.vaadin.flow.i18n.LocaleChangeEvent;
import com.vaadin.flow.i18n.LocaleChangeObserver;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.server.VaadinSession;
import com.vaadin.flow.server.auth.AnonymousAllowed;
import io.jmix.core.CoreProperties;
import io.jmix.core.MessageTools;
import io.jmix.core.security.AccessDeniedException;
import io.jmix.core.security.CurrentAuthentication;
import io.jmix.flowui.UiComponents;
import io.jmix.flowui.app.main.StandardMainView;
import io.jmix.flowui.component.main.JmixUserIndicator;
import io.jmix.flowui.component.select.JmixSelect;
import io.jmix.flowui.kit.component.ComponentUtils;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.view.Subscribe;
import io.jmix.flowui.view.ViewComponent;
import io.jmix.flowui.view.ViewController;
import io.jmix.flowui.view.ViewDescriptor;
import io.jmix.securityflowui.authentication.AuthDetails;
import io.jmix.securityflowui.authentication.LoginViewSupport;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;

import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.function.Function;
import java.util.stream.Collectors;

@Route("")
@ViewController("MainView")
@ViewDescriptor("main-view.xml")
public class MainView extends StandardMainView implements LocaleChangeObserver {

    private static final Logger log = LoggerFactory.getLogger(MainView.class);

    @ViewComponent
    private IFrame mainIFrame;

    @Autowired
    protected CoreProperties coreProperties;
    @Autowired
    protected MessageTools messageTools;
    @Autowired
    protected LoginViewSupport loginViewSupport;
    @Autowired
    protected UiComponents uiComponents;

    @ViewComponent
    protected JmixSelect<Locale> userLocale;
    @ViewComponent
    protected JmixButton loginButton;
    @ViewComponent
    protected JmixButton logoutButton;
    @Autowired
    protected CurrentAuthentication currentAuthentication;
    @ViewComponent
    protected JmixUserIndicator userIndicator;

    protected LoginOverlay loginOverlay;

    protected boolean localeInitialized;

    @Subscribe
    public void onInit(final InitEvent event) {
        mainIFrame.setSrc("img/anonymous-access.html");

        initLoginOverlay();
        initLocales();
    }

    @Subscribe
    public void onReady(ReadyEvent event) {
        initComponentsVisibility();
        localeInitialized = true;
    }

    protected void initLoginOverlay() {
        loginOverlay = uiComponents.create(LoginOverlay.class);
        getContent().addToDrawer(loginOverlay);

        loginOverlay.setForgotPasswordButtonVisible(false);
        loginOverlay.addLoginListener(this::onLogin);
    }

    protected void initComponentsVisibility() {
        if ("anonymous".equals(currentAuthentication.getUser().getUsername())) {
            userLocale.setVisible(true);
            loginButton.setVisible(true);
        } else {
            userIndicator.setVisible(true);
            logoutButton.setVisible(true);
        }
    }

    protected void initLocales() {
        LinkedHashMap<Locale, String> locales = coreProperties.getAvailableLocales().stream()
                .collect(Collectors.toMap(Function.identity(), messageTools::getLocaleDisplayName, (s1, s2) -> s1,
                        LinkedHashMap::new));

        ComponentUtils.setItemsMap(userLocale, locales);

        userLocale.setValue(VaadinSession.getCurrent().getLocale());
    }

    @Subscribe("userLocale")
    public void onUserLocaleComponentValueChange(final ComponentValueChangeEvent<JmixSelect<Locale>, Locale> event) {
        if (event.getOldValue() == null) {
            // do nothing
            // initial setup
            return;
        }

        VaadinSession.getCurrent().setLocale(event.getValue());
    }

    @Override
    public void localeChange(LocaleChangeEvent event) {
        if (localeInitialized) {
            UI.getCurrent().getPage().reload();
        }
    }

    @Subscribe(id = "loginButton", subject = "clickListener")
    public void onLoginButtonClick(final ClickEvent<JmixButton> event) {
        loginOverlay.setOpened(true);
    }

    protected void onLogin(AbstractLogin.LoginEvent event) {
        try {
            loginViewSupport.authenticate(
                    AuthDetails.of(event.getUsername(), event.getPassword())
                            // select your locale here
                            .withLocale(userLocale.getValue())
                            .withRememberMe(false)
            );
            onLoginSuccess();
        } catch (final BadCredentialsException | DisabledException | LockedException | AccessDeniedException e) {
            log.warn("Login failed for user '{}': {}", event.getUsername(), e.toString());
        } finally {
            ((LoginOverlay) event.getSource()).close();
        }
    }

    protected void onLoginSuccess() {
        UI.getCurrent().getPage().reload();
    }
}
